/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.aggraxl.VewAXL
 *
 * de.inovat.dua.aggraxl.VewAXL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.aggraxl.VewAXL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.aggraxl.VewAXL; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.aggraxl;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.archive.*;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.sys.funclib.debug.Debug;

//~ JDK IMPORTE ===============================================================

import java.io.IOException;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Map;
import java.util.TreeMap;

//~ KLASSEN ===================================================================

/**
 * Verwaltungsklasse je Aggregationsobjekt (DeAxl), welches für genau ein Objekt die Aggregation für einen Auftrag
 * ausführt.
 * <p>
 * Das Verwaltungsobjekt fragt sowohl die Archivdaten ab, berechnet die Aggreagationsdaten, und schreibt das Ergebnis
 * zurück in den Datenverteiler.
 * <p>
 * Weiterhin gibt die Verwaltungsklasse eine aussagekräftige Meldung zurück, die den vollständigen Ablauf der
 * Aggregation beschreibt.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class VewAXL implements ClientSenderInterface {

    /** DebugLogger für Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    /** Datumsformattierer  zur Ausgabe von Datum und Uhrzeit. */
    private static SimpleDateFormat datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    /** Archivsystem */
    private static ArchiveRequestManager _archiv;
    private static Aspect                _asp;
    private static AttributeGroup        _atg;
    private static DataDescription       _datenBeschreibung;

    /** Datenverteilerverbindung */
    private static ClientDavInterface _dav;

    /** Die Konfiguration des DaV. */
    private static DataModel _konfiguration;

    //~ FELDER ================================================================

    /** Das Systemobjekt, für welches diese Verwaltungsklasse die Daten berechnet */
    SystemObject _axlDe;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor der Verwaltungs Klasse.
     *
     * @param dav   Die Datenverteilerverbindung
     * @param axlDe Das Systemobjekt, für welches diese Verwaltungsklasse die Daten berechnen soll
     */
    public VewAXL(ClientDavInterface dav, SystemObject axlDe) {
        _dav               = dav;
        _axlDe             = axlDe;
        _konfiguration     = _dav.getDataModel();
        _archiv            = _dav.getArchive();
        _atg               = _konfiguration.getAttributeGroup("atg.tlsAxlErgebnisMeldungAchsLastDatenVersion11");
        _asp               = _konfiguration.getAspect("asp.tlsAntwort");
        _datenBeschreibung = new DataDescription(_atg, _asp);

        // Sendeanmeldung für Zielattributgruppe
        try {
            _dav.subscribeSender(this, _axlDe, _datenBeschreibung, SenderRole.source());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.warning(String.format("Doppelanmeldung für [%s] mit Datenbeschreibung [%s] (andere Quelle vorhanden?). Statistikdaten werden für Objekt nicht weiter berücksichtigt.", _axlDe.getPid(), _datenBeschreibung), e);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Methode, die den eigentlichen Auftrag ausführt.
     *
     * @param auftrag Der auszuführende Auftrag.
     *
     * @return Meldung, die den Ablauf der Ausführung des Auftrags enthält. Wird für die Statusdarstellung verwendet.
     */
    public String ausfuehren(Auftrag auftrag) {

        // Map der aktuellen Datensätze für das zu berücksichtigende Intervall (Schlüssel = Datenzeitstempel, Wert = Archivdatensatz)
        Map<Long, ArchiveData> _mapArchivIntervallDaten = new TreeMap<Long, ArchiveData>();

        // Meldungstext für die Auftragsabwicklung (wird über DaV als Status ausgegeben)
        StringBuffer meldung = new StringBuffer();

        meldung.append(auftrag.toString());

        ArchiveDataSpecification anfrage = null;
        ArchiveDataQueryResult   archivAnfrageErgebnis;
        ArchiveDataStream        archivStream[];
        ArchiveData              ad;

        // Archivdaten abfragen
        ArchiveTimeSpecification   zeitSpezifikation = new ArchiveTimeSpecification(TimingType.DATA_TIME, false, auftrag.getIntervallBeginn(), auftrag.getIntervallEnde() - 1);
        ArchiveDataKindCombination archivDatenTypen  = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED);

        // Abhängig vom Aufrufparameter die Version3 oder Version1Bis2 verwenden (eigeführt für BW-DAUZ)
        DataDescription datenBeschreibung;

        switch (AggrAXL.getVersionAxlEinzeldaten()) {
        case 1 :
        case 2 :
            datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion1Bis2"), _konfiguration.getAspect("asp.tlsAntwort"));

            break;

        case 3 :
            datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion3"), _konfiguration.getAspect("asp.tlsAntwort"));

            break;
        default:
	        datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion3"), _konfiguration.getAspect("asp.tlsAntwort"));
        }

        try {
            anfrage               = new ArchiveDataSpecification(zeitSpezifikation, archivDatenTypen, ArchiveOrder.BY_INDEX, ArchiveRequestOption.NORMAL, datenBeschreibung, _axlDe);
            archivAnfrageErgebnis = _archiv.request(ArchiveQueryPriority.LOW, anfrage);
            archivStream          = archivAnfrageErgebnis.getStreams();

            // Alle Daten einsortieren nach Datenzeitstempel. Dadurch werden bei doppelten Datensätzen immer nur die neusten berücksichtigt.
            while ((ad = archivStream[0].take()) != null) {
                if (ad.getDataType().equals(DataState.DATA)) {
                    _mapArchivIntervallDaten.put(ad.getDataTime(), ad);
                    debug.fine(String.format("%s [%s] (%d) %s-%s [%s] : %s%n", ad.getDataType(), ad.getDataKind(), ad.getDataIndex(), datumsFormatierer.format(new Date(ad.getDataTime())), datumsFormatierer.format(new Date(ad.getArchiveTime())), ad.getObject(), ad.getData()));
                } else {
                    debug.fine(String.format("%s : %s%n", ad.getDataType(), datumsFormatierer.format(new Date(ad.getDataTime()))));
                }
            }
        }
        catch (InterruptedException e) {
            meldung.append(String.format("%nArchivantwort auf Archivanfrage [%s] konnte nicht nicht vollständig ermittelt werden (unerwartete Unterbrechung bei der Übertragung der Antwort).", anfrage));
            debug.warning(String.format("Archivantwort auf Archivanfrage [%s] konnte nicht nicht vollständig ermittelt werden (unerwartete Unterbrechung bei der Übertragung der Antwort).", anfrage), e);
        }
        catch (IllegalStateException e) {
            meldung.append(String.format("Archivantwort auf Archivanfrage [%s] konnte nicht ermittelt werden (Keine Ergebnisdaten ermittelbar).", anfrage));
            debug.warning(String.format("Archivantwort auf Archivanfrage [%s] konnte nicht ermittelt werden (Keine Ergebnisdaten ermittelbar).", anfrage), e);
        }
        catch (IOException e) {
            meldung.append(String.format("Archivantwort auf Archivanfrage [%s] konnte nicht vollständig ermittelt werden (Probleme in der Kommunikation mit dem Archivsystem aufgetreten. Nicht alle Datensätze wurden übertragen).", anfrage));
            debug.warning(String.format("Archivantwort auf Archivanfrage [%s] konnte nicht vollständig ermittelt werden (Probleme in der Kommunikation mit dem Archivsystem aufgetreten. Nicht alle Datensätze wurden übertragen).", anfrage), e);
        }

        //
        // Archivdaten für das Intervall aggregrieren
        ResultData ergebnisDaten = berechneAchslastDatenVersion11(auftrag, _mapArchivIntervallDaten);

        // Aggregierte Daten an DaV senden
        try {
            _dav.sendData(ergebnisDaten);
        }
        catch (SendSubscriptionNotConfirmed e) {
            meldung.append(String.format("Sendeanmeldung für [%s] mit Datenbeschreibung [%s] noch nicht erfolgt. Statistikdaten konnten nicht an den Datenverteiler gesendet werden.", _axlDe.getPid(), datenBeschreibung));
            debug.warning(String.format("Sendeanmeldung für [%s] mit Datenbeschreibung [%s] noch nicht erfolgt. Statistikdaten konnten nicht an den Datenverteiler gesendet werden.", _axlDe.getPid(), datenBeschreibung), e);
        }

        // Meldung mit dem Ergebnis für die Debugausgabe bzw. für die Fortschrittsanzeige zurückgeben.
        return meldung.toString();
    }

    /**
     * Methode, die aus den spontanen Archivdaten die Intervalldaten für die Achslast Version 11 Berechnet
     *
     * @param auftrag                 Der Auftrag, für den die Intervalldaten berechnet werden sollen.
     * @param mapArchivIntervallDaten Die Map mit den spontan gelieferten Einzel Fahrzeugdaten, aus denen die
     *                                Intervalldaten berechnet werden sollen
     *
     * @return Der Ergebnisdatensatz mit den berechneten Intervalldaten.
     */
    private ResultData berechneAchslastDatenVersion11(Auftrag auftrag, Map<Long, ArchiveData> mapArchivIntervallDaten) {
        Data       daten = _dav.createData(_atg);
        Data.Array da;
        Parameter  p = Parameter.getInstanz();
        int        anz, i;
        int[]      anzEinzelachslastKlasse;
        int[]      anzDoppelachslastKlasse;
        int[]      anzDreifachachslastKlasse;
        int        anzUeberladungLkw       = 0;
        int        anzUeberladungBusse     = 0;
        int        anzUeberladungLkwA      = 0;
        int        anzUeberladungSattelKfz = 0;
        int[]      anzGewichtsKlasseLkw;
        int[]      anzGewichtsKlasseBusse;
        int[]      anzGewichtsKlasseLkwA;
        int[]      anzGewichtsKlasseSattelKfz;

        // Daten initialisieren
        daten.getTimeValue("T").setSeconds((auftrag.getIntervallEnde() - auftrag.getIntervallBeginn()) / 1000);

        // Einzelachsen vorbelegen
        anz                     = p.getGrenzenEinzelAchsen().length + 1;
        anzEinzelachslastKlasse = new int[anz];
        da                      = daten.getArray("AnzEinzelachslastKlasse");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzEinzelachslastKlasse[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Doppelachsen vorbelegen
        anz                     = p.getGrenzenDoppelAchsen().length + 1;
        anzDoppelachslastKlasse = new int[anz];
        da                      = daten.getArray("AnzDoppelachslastKlasse");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzDoppelachslastKlasse[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Dreifachachsen vorbelegen
        anz                       = p.getGrenzenDreifachAchsen().length + 1;
        anzDreifachachslastKlasse = new int[anz];
        da                        = daten.getArray("AnzDreifachachslastKlasse");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzDreifachachslastKlasse[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Anzahlen für Überladungen
        daten.getUnscaledValue("AnzÜberladungLkw").set(0);
        daten.getUnscaledValue("AnzÜberladungBusse").set(0);
        daten.getUnscaledValue("AnzÜberladungLkwA").set(0);
        daten.getUnscaledValue("AnzÜberladungSattelKfz").set(0);

        // Gesamtgewichte Klasse 3 vorbelegen
        anz                  = p.getGrenzenFzKlasse3().length + 1;
        anzGewichtsKlasseLkw = new int[anz];
        da                   = daten.getArray("AnzGewichtsKlasseLkw");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzGewichtsKlasseLkw[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Gesamtgewichte Klasse 5 vorbelegen
        anz                    = p.getGrenzenFzKlasse5().length + 1;
        anzGewichtsKlasseBusse = new int[anz];
        da                     = daten.getArray("AnzGewichtsKlasseBusse");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzGewichtsKlasseBusse[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Gesamtgewichte Klasse 8 vorbelegen
        anz                   = p.getGrenzenFzKlasse8().length + 1;
        anzGewichtsKlasseLkwA = new int[anz];
        da                    = daten.getArray("AnzGewichtsKlasseLkwA");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzGewichtsKlasseLkwA[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Gesamtgewichte Klasse 9 vorbelegen
        anz                        = p.getGrenzenFzKlasse9().length + 1;
        anzGewichtsKlasseSattelKfz = new int[anz];
        da                         = daten.getArray("AnzGewichtsKlasseSattelKfz");
        da.setLength(anz);

        for (i = 0; i < anz; i++) {
            anzGewichtsKlasseSattelKfz[i] = 0;
            da.getUnscaledValue(i).set(0);
        }

        // Jetzt die eigentlichen Daten aggregieren
        for (ArchiveData ad : mapArchivIntervallDaten.values()) {

            // Anzahl AnzGewichtsKlasseXXX und AnzÜberladungXXX aus folgenden Angaben ermitteln...
            int     gewichtsKlasse;
            int     fzTyp           = ad.getData().getUnscaledValue("FzTyp").intValue();
            int     fzKlasse        = ermittleFzKlasse(fzTyp);
            long    fzGesamtGewicht = ad.getData().getUnscaledValue("FzGesamtGewicht").longValue();
            boolean fzUeberladung   = ad.getData().getUnscaledValue("FzÜberladung").intValue() != 0;

            switch (fzKlasse) {
            case 3 :  // LKW
                gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenFzKlasse3(), fzGesamtGewicht);
                anzGewichtsKlasseLkw[gewichtsKlasse]++;

                if (fzUeberladung) {
                    anzUeberladungLkw++;
                }

                break;

            case 5 :  // Busse
                gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenFzKlasse5(), fzGesamtGewicht);
                anzGewichtsKlasseBusse[gewichtsKlasse]++;

                if (fzUeberladung) {
                    anzUeberladungBusse++;
                }

                break;

            case 8 :  // LkwA
                gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenFzKlasse8(), fzGesamtGewicht);
                anzGewichtsKlasseLkwA[gewichtsKlasse]++;

                if (fzUeberladung) {
                    anzUeberladungLkwA++;
                }

                break;

            case 9 :  // SattelKfz
                gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenFzKlasse9(), fzGesamtGewicht);
                anzGewichtsKlasseSattelKfz[gewichtsKlasse]++;

                if (fzUeberladung) {
                    anzUeberladungSattelKfz++;
                }

                break;

            default :

            // PKWs werden nicht berücksichtigt
            }

            // Anzahl der Achslastklassen AnzXXXachslastKlasse aus den Achslastdaten ermitteln...
            // Schleife über die einzelen Achsen
            da = ad.getData().getArray("ÜberlastungAchse");

            for (i = 0; i < da.getLength(); i++) {
                if (da.getItem(i).getUnscaledValue("ÜberlastEinzelachse").intValue() == 1) {
                    gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenEinzelAchsen(), da.getItem(i).getUnscaledValue("Achslast").longValue());
                    anzEinzelachslastKlasse[gewichtsKlasse]++;

                    break;
                }

                if (da.getItem(i).getUnscaledValue("ÜberlastDoppelachse").intValue() == 1) {
                    gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenDoppelAchsen(), da.getItem(i).getUnscaledValue("Achslast").longValue());
                    anzDoppelachslastKlasse[gewichtsKlasse]++;

                    break;
                }

                if (da.getItem(i).getUnscaledValue("ÜberlastDreifachachse").intValue() == 1) {
                    gewichtsKlasse = ermittleGewichtsKlasse(p.getGrenzenDreifachAchsen(), da.getItem(i).getUnscaledValue("Achslast").longValue());
                    anzDreifachachslastKlasse[gewichtsKlasse]++;

                    break;
                }
            }
        }

        // Datensatz zusammenstellen...
        // Einzelachsen setzen
        anz = p.getGrenzenEinzelAchsen().length + 1;
        da  = daten.getArray("AnzEinzelachslastKlasse");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzEinzelachslastKlasse[i]);
        }

        // Doppelachsen setzen
        anz = p.getGrenzenDoppelAchsen().length + 1;
        da  = daten.getArray("AnzDoppelachslastKlasse");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzDoppelachslastKlasse[i]);
        }

        // Dreifachachsen setzen
        anz = p.getGrenzenDreifachAchsen().length + 1;
        da  = daten.getArray("AnzDreifachachslastKlasse");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzDreifachachslastKlasse[i]);
        }

        // Anzahlen für Überladungen setzen
        daten.getUnscaledValue("AnzÜberladungLkw").set((anzUeberladungLkw < 254) ? anzUeberladungLkw : 254);
        daten.getUnscaledValue("AnzÜberladungBusse").set((anzUeberladungBusse < 254) ? anzUeberladungBusse : 254);
        daten.getUnscaledValue("AnzÜberladungLkwA").set((anzUeberladungLkwA < 254) ? anzUeberladungLkwA : 254);
        daten.getUnscaledValue("AnzÜberladungSattelKfz").set((anzUeberladungSattelKfz < 254) ? anzUeberladungSattelKfz : 254);

        // Gesamtgewichte Klasse 3 setzen
        anz = p.getGrenzenFzKlasse3().length + 1;
        da  = daten.getArray("AnzGewichtsKlasseLkw");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzGewichtsKlasseLkw[i]);
        }

        // Gesamtgewichte Klasse 5 setzen
        anz = p.getGrenzenFzKlasse5().length + 1;
        da  = daten.getArray("AnzGewichtsKlasseBusse");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzGewichtsKlasseBusse[i]);
        }

        // Gesamtgewichte Klasse 8 setzen
        anz = p.getGrenzenFzKlasse8().length + 1;
        da  = daten.getArray("AnzGewichtsKlasseLkwA");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzGewichtsKlasseLkwA[i]);
        }

        // Gesamtgewichte Klasse 9 setzen
        anz = p.getGrenzenFzKlasse9().length + 1;
        da  = daten.getArray("AnzGewichtsKlasseSattelKfz");

        for (i = 0; i < anz; i++) {
            da.getUnscaledValue(i).set(anzGewichtsKlasseSattelKfz[i]);
        }

        return new ResultData(_axlDe, _datenBeschreibung, auftrag.getIntervallBeginn(), daten);
    }

    /**
     * Sendesteuerung des Datenverteilers an die Applikation. Diese Methode muss von der Applikation implementiert
     * werden, um den Versand von Daten zu starten bzw. anzuhalten. Der Datenverteiler signalisiert damit einer Quelle
     * oder einem Sender dass mindestens ein Abnehmer bzw. kein Abnehmer mehr für die zuvor angemeldeten Daten vorhanden
     * ist. Die Quelle wird damit aufgefordert den Versand von Daten zu starten bzw. zu stoppen.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung
     *                        bezieht.
     * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung
     *                        bezieht.
     * @param state           Status der Sendesteuerung. Kann einen der Werte <code>START_SENDING</code>,
     *                        <code>STOP_SENDING</code>, <code>STOP_SENDING_NO_RIGHTS</code>,
     *                        <code>STOP_SENDING_NOT_A_VALID_SUBSCRIPTION</code> enthalten.
     *
     * @see #START_SENDING
     * @see #STOP_SENDING
     * @see #STOP_SENDING_NO_RIGHTS
     * @see #STOP_SENDING_NOT_A_VALID_SUBSCRIPTION
     */
    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {

        // Wird nicht unterstützt. Ergebnisdaten werden immer gesendet.
    }

    /**
     * Hilfsfunktion, welche aus dem Fahrzeugtyp die Fahrzeugklasse nach TLS ermittelt
     *
     * @param fzTyp Der Fahrzeugtyp gemäß TLS
     *
     * @return Die Fahrzeugklasse gemäß TLS
     */
    private int ermittleFzKlasse(int fzTyp) {
        if ((8 <= fzTyp) && (fzTyp <= 12)) {
            return 3;
        }

        if ((120 <= fzTyp) && (fzTyp <= 125)) {
            return 5;
        }

        if ((32 <= fzTyp) && (fzTyp <= 70)) {
            return 8;
        }

        if ((96 <= fzTyp) && (fzTyp <= 107)) {
            return 9;
        }

        return 0;
    }

    /**
     * in Hilfsfunktion,  welche das Fahrzeuggewicht einer Gewichtsklasse zugeordnet
     *
     * @param grenzenGewichtsKlassen Array mit den Gewichtsklassen
     * @param fzGesamtGewicht        Das Fahrzeug Gesamtgewicht, welches zugeordnet werden soll
     *
     * @return Die ermittelte Gewichtsklasse für das Fahrzeug (Index)
     */
    private int ermittleGewichtsKlasse(long[] grenzenGewichtsKlassen, long fzGesamtGewicht) {
        for (int i = 0; i < grenzenGewichtsKlassen.length; i++) {
            if (fzGesamtGewicht < grenzenGewichtsKlassen[i]) {
                return i;
            }
        }

        return grenzenGewichtsKlassen.length;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Pid des dieser Verwaltungseinheit zugeordnete Achslast-DE Systemobjekt.
     *
     * @return Pid des dieser Verwaltungseinheit zugeordnete Achslast-DE Systemobjekt.
     */
    public String getPid() {
        return _axlDe.getPid();
    }

    /**
     * Diese Methode muss von der Applikation implementiert werden, um zu signalisieren, ob Sendesteuerungen erwünscht
     * sind und mit der Methode <code>dataRequest</code> verarbeitet werden. In der Implementierung dieser Methode
     * dürfen keine synchronen Aufrufe, die auf Telegramme vom Datenverteiler warten (wie z.B. Konfigurationsanfragen)
     * durchgeführt werden, da ansonsten ein Deadlock entsteht.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene System-Objekt.
     * @param dataDescription Die in der zugehörigen Sendeanmeldung angegebenen beschreibenden Informationen der
     *                        angemeldeten Daten.
     *
     * @return <code>true</code>, falls Sendesteuerungen gewünscht sind, sonst <code>false</code>.
     *
     * @see #dataRequest
     */
    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 02.09.14
